@@ -730,7 +730,9 @@ meta_theme_gtk_draw_frame (MetaThemeImpl           *impl,
   if (style->layout->has_title && title_layout)
     {
       PangoRectangle logical;
-      gdouble text_width, x, y;
+      gdouble text_width;
+      int x;
+      int y;
 
       pango_layout_set_width (title_layout, -1);
       pango_layout_get_pixel_extents (title_layout, NULL, &logical);
@@ -876,6 +878,7 @@ meta_theme_gtk_draw_frame (MetaThemeImpl           *impl,
 
               g_assert (pixbuf == NULL);
               pixbuf = gtk_icon_info_load_symbolic_for_context (info, context, NULL, NULL);
+              g_object_unref (info);
             }
 
           if (pixbuf)
