@@ -35,11 +35,20 @@ struct _MetaTheme
   MetaThemeType         type;
   MetaThemeImpl        *impl;
 
+  gulong                gtk_theme_name_id;
+  gulong                gtk_xft_antialias_id;
+  gulong                gtk_xft_hinting_id;
+  gulong                gtk_xft_hintstyle_id;
+  gulong                gtk_xft_rgba_id;
+
+  MetaButtonLayout     *button_layout;
+
   gboolean              composited;
+  gint                  scale;
+  gdouble               dpi;
 
   PangoFontDescription *titlebar_font;
 
-  gulong                gtk_theme_name_id;
   gchar                *gtk_theme_name;
   GHashTable           *variants;
 
@@ -62,6 +71,34 @@ static GParamSpec *properties[LAST_PROP] = { NULL };
 
 G_DEFINE_TYPE (MetaTheme, meta_theme, G_TYPE_OBJECT)
 
+static void
+update_composited_func (gpointer key,
+                        gpointer value,
+                        gpointer user_data)
+{
+  MetaTheme *theme;
+  MetaStyleInfo *style_info;
+
+  theme = META_THEME (user_data);
+  style_info = META_STYLE_INFO (value);
+
+  meta_style_info_set_composited (style_info, theme->composited);
+}
+
+static void
+update_scale_func (gpointer key,
+                   gpointer value,
+                   gpointer user_data)
+{
+  MetaTheme *theme;
+  MetaStyleInfo *style_info;
+
+  theme = META_THEME (user_data);
+  style_info = META_STYLE_INFO (value);
+
+  meta_style_info_set_scale (style_info, theme->scale);
+}
+
 static MetaStyleInfo *
 get_style_info (MetaTheme   *theme,
                 const gchar *variant)
@@ -77,11 +114,8 @@ get_style_info (MetaTheme   *theme,
 
   if (style_info == NULL)
     {
-      gint window_scale;
-
-      window_scale = get_window_scaling_factor ();
       style_info = meta_style_info_new (theme->gtk_theme_name, variant,
-                                        theme->composited, window_scale);
+                                        theme->composited, theme->scale);
 
       g_hash_table_insert (theme->variants, g_strdup (key), style_info);
     }
@@ -89,97 +123,6 @@ get_style_info (MetaTheme   *theme,
   return style_info;
 }
 
-static MetaFrameStyle *
-get_frame_style (MetaTheme      *theme,
-                 MetaFrameType   type,
-                 MetaFrameFlags  flags)
-{
-  MetaFrameState state;
-  MetaFrameResize resize;
-  MetaFrameFocus focus;
-  MetaFrameStyle *style;
-  MetaFrameStyleSet *style_set;
-
-  g_return_val_if_fail (type < META_FRAME_TYPE_LAST, NULL);
-
-  style_set = meta_theme_impl_get_style_set (theme->impl, type);
-
-  if (style_set == NULL && type == META_FRAME_TYPE_ATTACHED)
-    style_set = meta_theme_impl_get_style_set (theme->impl, META_FRAME_TYPE_BORDER);
-
-  /* Right now the parser forces a style set for all other types,
-   * but this fallback code is here in case I take that out.
-   */
-  if (style_set == NULL)
-    style_set = meta_theme_impl_get_style_set (theme->impl, META_FRAME_TYPE_NORMAL);
-
-  if (style_set == NULL)
-    return NULL;
-
-  switch (flags & (META_FRAME_MAXIMIZED | META_FRAME_SHADED | META_FRAME_TILED_LEFT | META_FRAME_TILED_RIGHT))
-    {
-    case 0:
-      state = META_FRAME_STATE_NORMAL;
-      break;
-    case META_FRAME_MAXIMIZED:
-      state = META_FRAME_STATE_MAXIMIZED;
-      break;
-    case META_FRAME_TILED_LEFT:
-      state = META_FRAME_STATE_TILED_LEFT;
-      break;
-    case META_FRAME_TILED_RIGHT:
-      state = META_FRAME_STATE_TILED_RIGHT;
-      break;
-    case META_FRAME_SHADED:
-      state = META_FRAME_STATE_SHADED;
-      break;
-    case (META_FRAME_MAXIMIZED | META_FRAME_SHADED):
-      state = META_FRAME_STATE_MAXIMIZED_AND_SHADED;
-      break;
-    case (META_FRAME_TILED_LEFT | META_FRAME_SHADED):
-      state = META_FRAME_STATE_TILED_LEFT_AND_SHADED;
-      break;
-    case (META_FRAME_TILED_RIGHT | META_FRAME_SHADED):
-      state = META_FRAME_STATE_TILED_RIGHT_AND_SHADED;
-      break;
-    default:
-      g_assert_not_reached ();
-      state = META_FRAME_STATE_LAST; /* compiler */
-      break;
-    }
-
-  switch (flags & (META_FRAME_ALLOWS_VERTICAL_RESIZE | META_FRAME_ALLOWS_HORIZONTAL_RESIZE))
-    {
-    case 0:
-      resize = META_FRAME_RESIZE_NONE;
-      break;
-    case META_FRAME_ALLOWS_VERTICAL_RESIZE:
-      resize = META_FRAME_RESIZE_VERTICAL;
-      break;
-    case META_FRAME_ALLOWS_HORIZONTAL_RESIZE:
-      resize = META_FRAME_RESIZE_HORIZONTAL;
-      break;
-    case (META_FRAME_ALLOWS_VERTICAL_RESIZE | META_FRAME_ALLOWS_HORIZONTAL_RESIZE):
-      resize = META_FRAME_RESIZE_BOTH;
-      break;
-    default:
-      g_assert_not_reached ();
-      resize = META_FRAME_RESIZE_LAST; /* compiler */
-      break;
-    }
-
-  /* re invert the styles used for focus/unfocussed while flashing a frame */
-  if (((flags & META_FRAME_HAS_FOCUS) && !(flags & META_FRAME_IS_FLASHING))
-      || (!(flags & META_FRAME_HAS_FOCUS) && (flags & META_FRAME_IS_FLASHING)))
-    focus = META_FRAME_FOCUS_YES;
-  else
-    focus = META_FRAME_FOCUS_NO;
-
-  style = meta_frame_style_set_get_style (style_set, state, resize, focus);
-
-  return style;
-}
-
 static void
 font_desc_apply_scale (PangoFontDescription *font_desc,
                        MetaTheme            *theme,
@@ -188,14 +131,13 @@ font_desc_apply_scale (PangoFontDescription *font_desc,
 {
   gint old_size;
   MetaFrameStyle *style;
-  gdouble scale;
   gint new_size;
 
   old_size = pango_font_description_get_size (font_desc);
-  style = get_frame_style (theme, type, flags);
-  scale = get_window_scaling_factor ();
+  style = META_THEME_IMPL_GET_CLASS (theme->impl)->get_frame_style (theme->impl,
+                                                                    type, flags);
 
-  new_size = MAX (old_size * (style->layout->title_scale / scale), 1);
+  new_size = MAX (old_size * (style->layout->title_scale / theme->scale), 1);
 
   pango_font_description_set_size (font_desc, new_size);
 }
@@ -261,7 +203,7 @@ ensure_pango_context (MetaTheme *theme)
   options = gdk_screen_get_font_options (screen);
   pango_cairo_context_set_font_options (context, options);
 
-  dpi = gdk_screen_get_resolution (screen);
+  dpi = theme->dpi * theme->scale;
   pango_cairo_context_set_resolution (context, dpi);
 
   theme->context = context;
@@ -294,7 +236,6 @@ get_title_height (MetaTheme      *theme,
       PangoFontMetrics *metrics;
       gint ascent;
       gint descent;
-      gint scale;
 
       ensure_pango_context (theme);
 
@@ -306,9 +247,7 @@ get_title_height (MetaTheme      *theme,
       pango_font_metrics_unref (metrics);
 
       title_height = PANGO_PIXELS (ascent + descent);
-      scale = get_window_scaling_factor ();
-
-      title_height *= scale;
+      title_height *= theme->scale;
 
       height = GINT_TO_POINTER (title_height);
       g_hash_table_insert (theme->title_heights, size, height);
@@ -344,6 +283,14 @@ create_title_layout (MetaTheme      *theme,
   return layout;
 }
 
+static void
+update_font_options (MetaTheme *theme)
+{
+  g_clear_object (&theme->context);
+  g_hash_table_remove_all (theme->font_descs);
+  g_hash_table_remove_all (theme->title_heights);
+}
+
 static void
 notify_gtk_theme_name_cb (GtkSettings *settings,
                           GParamSpec  *pspec,
@@ -355,14 +302,49 @@ notify_gtk_theme_name_cb (GtkSettings *settings,
   meta_theme_invalidate (theme);
 }
 
+static void
+notify_gtk_xft_antialias_cb (GtkSettings *settings,
+                             GParamSpec  *pspec,
+                             MetaTheme   *theme)
+{
+  update_font_options (theme);
+}
+
+static void
+notify_gtk_xft_hinting_cb (GtkSettings *settings,
+                           GParamSpec  *pspec,
+                           MetaTheme   *theme)
+{
+  update_font_options (theme);
+}
+
+static void
+notify_gtk_xft_hintstyle_cb (GtkSettings *settings,
+                             GParamSpec  *pspec,
+                             MetaTheme   *theme)
+{
+  update_font_options (theme);
+}
+
+static void
+notify_gtk_xft_rgba_cb (GtkSettings *settings,
+                        GParamSpec  *pspec,
+                        MetaTheme   *theme)
+{
+  update_font_options (theme);
+}
+
 static void
 meta_theme_constructed (GObject *object)
 {
   MetaTheme *theme;
+  GtkSettings *settings;
+  const gchar *button_layout;
 
   G_OBJECT_CLASS (meta_theme_parent_class)->constructed (object);
 
   theme = META_THEME (object);
+  settings = gtk_settings_get_default ();
 
   if (theme->type == META_THEME_TYPE_GTK)
     theme->impl = g_object_new (META_TYPE_THEME_GTK, NULL);
@@ -372,27 +354,76 @@ meta_theme_constructed (GObject *object)
     g_assert_not_reached ();
 
   meta_theme_impl_set_composited (theme->impl, theme->composited);
+  meta_theme_impl_set_scale (theme->impl, theme->scale);
+
+  button_layout = "appmenu:minimize,maximize,close";
+  meta_theme_set_button_layout (theme, button_layout, FALSE);
+
+  if (theme->type == META_THEME_TYPE_METACITY)
+    {
+      theme->gtk_theme_name_id =
+        g_signal_connect (settings, "notify::gtk-theme-name",
+                          G_CALLBACK (notify_gtk_theme_name_cb), theme);
+    }
+
+  theme->gtk_xft_antialias_id =
+    g_signal_connect (settings, "notify::gtk-xft-antialias",
+                      G_CALLBACK (notify_gtk_xft_antialias_cb), theme);
+
+  theme->gtk_xft_hinting_id =
+    g_signal_connect (settings, "notify::gtk-xft-hinting",
+                      G_CALLBACK (notify_gtk_xft_hinting_cb), theme);
+
+  theme->gtk_xft_hintstyle_id =
+    g_signal_connect (settings, "notify::gtk-xft-hintstyle",
+                      G_CALLBACK (notify_gtk_xft_hintstyle_cb), theme);
+
+  theme->gtk_xft_rgba_id =
+    g_signal_connect (settings, "notify::gtk-xft-rgba",
+                      G_CALLBACK (notify_gtk_xft_rgba_cb), theme);
 }
 
 static void
 meta_theme_dispose (GObject *object)
 {
   MetaTheme *theme;
+  GtkSettings *settings;
 
   theme = META_THEME (object);
+  settings = gtk_settings_get_default ();
 
   g_clear_object (&theme->impl);
 
   if (theme->gtk_theme_name_id > 0)
     {
-      GtkSettings *settings;
-
-      settings = gtk_settings_get_default ();
-
       g_signal_handler_disconnect (settings, theme->gtk_theme_name_id);
       theme->gtk_theme_name_id = 0;
     }
 
+  if (theme->gtk_xft_antialias_id > 0)
+    {
+      g_signal_handler_disconnect (settings, theme->gtk_xft_antialias_id);
+      theme->gtk_xft_antialias_id = 0;
+    }
+
+  if (theme->gtk_xft_hinting_id > 0)
+    {
+      g_signal_handler_disconnect (settings, theme->gtk_xft_hinting_id);
+      theme->gtk_xft_hinting_id = 0;
+    }
+
+  if (theme->gtk_xft_hintstyle_id > 0)
+    {
+      g_signal_handler_disconnect (settings, theme->gtk_xft_hintstyle_id);
+      theme->gtk_xft_hintstyle_id = 0;
+    }
+
+  if (theme->gtk_xft_rgba_id > 0)
+    {
+      g_signal_handler_disconnect (settings, theme->gtk_xft_rgba_id);
+      theme->gtk_xft_rgba_id = 0;
+    }
+
   g_clear_pointer (&theme->variants, g_hash_table_destroy);
 
   g_clear_object (&theme->context);
@@ -410,6 +441,12 @@ meta_theme_finalize (GObject *object)
 
   theme = META_THEME (object);
 
+  if (theme->button_layout != NULL)
+    {
+      meta_button_layout_free (theme->button_layout);
+      theme->button_layout = NULL;
+    }
+
   if (theme->titlebar_font)
     {
       pango_font_description_free (theme->titlebar_font);
@@ -474,6 +511,8 @@ static void
 meta_theme_init (MetaTheme *theme)
 {
   theme->composited = TRUE;
+  theme->scale = 1;
+  theme->dpi = 96.0;
 
   theme->variants = g_hash_table_new_full (g_str_hash, g_str_equal,
                                            g_free, g_object_unref);
@@ -521,13 +560,6 @@ meta_theme_load (MetaTheme    *theme,
 
       g_free (theme->gtk_theme_name);
       g_object_get (settings, "gtk-theme-name", &theme->gtk_theme_name, NULL);
-
-      if (theme->gtk_theme_name_id == 0)
-        {
-          theme->gtk_theme_name_id =
-            g_signal_connect (settings, "notify::gtk-theme-name",
-                              G_CALLBACK (notify_gtk_theme_name_cb), theme);
-        }
     }
   else
     {
@@ -547,6 +579,130 @@ meta_theme_invalidate (MetaTheme *theme)
   g_hash_table_remove_all (theme->title_heights);
 }
 
+void
+meta_theme_set_button_layout (MetaTheme   *theme,
+                              const gchar *button_layout,
+                              gboolean     invert)
+{
+  if (theme->button_layout != NULL)
+    meta_button_layout_free (theme->button_layout);
+
+  theme->button_layout = meta_button_layout_new (button_layout, invert);
+}
+
+MetaButton *
+meta_theme_get_button (MetaTheme *theme,
+                       gint       x,
+                       gint       y)
+{
+  gint side;
+
+  for (side = 0; side < 2; side++)
+    {
+      MetaButton *buttons;
+      gint n_buttons;
+      gint i;
+
+      if (side == 0)
+        {
+          buttons = theme->button_layout->left_buttons;
+          n_buttons = theme->button_layout->n_left_buttons;
+        }
+      else if (side == 1)
+        {
+          buttons = theme->button_layout->right_buttons;
+          n_buttons = theme->button_layout->n_right_buttons;
+        }
+      else
+        {
+          g_assert_not_reached ();
+        }
+
+      for (i = 0; i < n_buttons; i++)
+        {
+          MetaButton *btn;
+          GdkRectangle rect;
+
+          btn = &buttons[i];
+          rect = btn->rect.visible;
+
+          if (!btn->visible || btn->type == META_BUTTON_TYPE_SPACER ||
+              rect.width <= 0 || rect.height <= 0)
+            {
+              continue;
+            }
+
+          rect = btn->rect.clickable;
+
+          if (x >= rect.x && x < (rect.x + rect.width) &&
+              y >= rect.y && y < (rect.y + rect.height))
+            {
+              return btn;
+            }
+        }
+    }
+
+  return NULL;
+}
+
+MetaButton **
+meta_theme_get_buttons (MetaTheme *theme)
+{
+  MetaButtonLayout *layout;
+  gint size;
+  MetaButton **retval;
+  gint index;
+  gint side;
+
+  layout = theme->button_layout;
+  size = layout->n_left_buttons + layout->n_right_buttons + 1;
+  retval = g_new0 (MetaButton *, size);
+  index = 0;
+
+  for (side = 0; side < 2; side++)
+    {
+      MetaButton *buttons;
+      gint n_buttons;
+      gint i;
+
+      if (side == 0)
+        {
+          buttons = layout->left_buttons;
+          n_buttons = layout->n_left_buttons;
+        }
+      else if (side == 1)
+        {
+          buttons = layout->right_buttons;
+          n_buttons = layout->n_right_buttons;
+        }
+      else
+        {
+          g_assert_not_reached ();
+        }
+
+      for (i = 0; i < n_buttons; i++)
+        {
+          MetaButton *btn;
+          GdkRectangle rect;
+
+          btn = &buttons[i];
+          rect = btn->rect.visible;
+
+          if (!btn->visible || btn->type == META_BUTTON_TYPE_SPACER ||
+              rect.width <= 0 || rect.height <= 0)
+            {
+              continue;
+            }
+
+          retval[index++] = btn;
+        }
+    }
+
+  retval[index] = NULL;
+
+  return retval;
+}
+
 void
 meta_theme_set_composited (MetaTheme *theme,
                            gboolean   composited)
@@ -557,7 +713,41 @@ meta_theme_set_composited (MetaTheme *theme,
   theme->composited = composited;
 
   meta_theme_impl_set_composited (theme->impl, composited);
-  meta_theme_invalidate (theme);
+  g_hash_table_foreach (theme->variants, update_composited_func, theme);
+
+  g_hash_table_remove_all (theme->font_descs);
+  g_hash_table_remove_all (theme->title_heights);
+}
+
+void
+meta_theme_set_dpi (MetaTheme *theme,
+                    gdouble    dpi)
+{
+  if (theme->dpi == dpi)
+    return;
+
+  theme->dpi = dpi;
+
+  g_clear_object (&theme->context);
+  g_hash_table_remove_all (theme->font_descs);
+  g_hash_table_remove_all (theme->title_heights);
+}
+
+void
+meta_theme_set_scale (MetaTheme *theme,
+                      gint       scale)
+{
+  if (theme->scale == scale)
+    return;
+
+  theme->scale = scale;
+
+  meta_theme_impl_set_scale (theme->impl, scale);
+  g_hash_table_foreach (theme->variants, update_scale_func, theme);
+
+  g_clear_object (&theme->context);
+  g_hash_table_remove_all (theme->font_descs);
+  g_hash_table_remove_all (theme->title_heights);
 }
 
 void
@@ -578,8 +768,8 @@ meta_theme_get_frame_borders (MetaTheme        *theme,
                               MetaFrameFlags    flags,
                               MetaFrameBorders *borders)
 {
-  MetaFrameStyle *style;
   MetaThemeImplClass *impl_class;
+  MetaFrameStyle *style;
   MetaStyleInfo *style_info;
   gint title_height;
 
@@ -587,13 +777,13 @@ meta_theme_get_frame_borders (MetaTheme        *theme,
 
   meta_frame_borders_clear (borders);
 
-  style = get_frame_style (theme, type, flags);
+  impl_class = META_THEME_IMPL_GET_CLASS (theme->impl);
+  style = impl_class->get_frame_style (theme->impl, type, flags);
 
   /* Parser is not supposed to allow this currently */
   if (style == NULL)
     return;
 
-  impl_class = META_THEME_IMPL_GET_CLASS (theme->impl);
   style_info = get_style_info (theme, variant);
   title_height = get_title_height (theme, variant, type, flags);
 
@@ -602,77 +792,124 @@ meta_theme_get_frame_borders (MetaTheme        *theme,
 }
 
 void
-meta_theme_calc_geometry (MetaTheme              *theme,
-                          const gchar            *variant,
-                          MetaFrameType           type,
-                          MetaFrameFlags          flags,
-                          gint                    client_width,
-                          gint                    client_height,
-                          const MetaButtonLayout *button_layout,
-                          MetaFrameGeometry      *fgeom)
+meta_theme_calc_geometry (MetaTheme         *theme,
+                          const gchar       *variant,
+                          MetaFrameType      type,
+                          MetaFrameFlags     flags,
+                          gint               client_width,
+                          gint               client_height,
+                          MetaFrameGeometry *fgeom)
 {
-  MetaFrameStyle *style;
   MetaThemeImplClass *impl_class;
+  MetaFrameStyle *style;
   MetaStyleInfo *style_info;
   gint title_height;
 
   g_return_if_fail (type < META_FRAME_TYPE_LAST);
 
-  style = get_frame_style (theme, type, flags);
+  impl_class = META_THEME_IMPL_GET_CLASS (theme->impl);
+  style = impl_class->get_frame_style (theme->impl, type, flags);
 
   /* Parser is not supposed to allow this currently */
   if (style == NULL)
     return;
 
-  impl_class = META_THEME_IMPL_GET_CLASS (theme->impl);
   style_info = get_style_info (theme, variant);
   title_height = get_title_height (theme, variant, type, flags);
 
   impl_class->calc_geometry (theme->impl, style->layout, style_info,
                              title_height, flags, client_width, client_height,
-                             button_layout, type, fgeom);
+                             theme->button_layout, type, fgeom);
 }
 
 void
-meta_theme_draw_frame (MetaTheme              *theme,
-                       const gchar            *variant,
-                       cairo_t                *cr,
-                       MetaFrameType           type,
-                       MetaFrameFlags          flags,
-                       gint                    client_width,
-                       gint                    client_height,
-                       const gchar            *title,
-                       const MetaButtonLayout *button_layout,
-                       MetaButtonState         button_states[META_BUTTON_TYPE_LAST],
-                       GdkPixbuf              *mini_icon,
-                       GdkPixbuf              *icon)
+meta_theme_draw_frame (MetaTheme           *theme,
+                       const gchar         *variant,
+                       cairo_t             *cr,
+                       MetaFrameType        type,
+                       MetaFrameFlags       flags,
+                       gint                 client_width,
+                       gint                 client_height,
+                       const gchar         *title,
+                       MetaButtonStateFunc  func,
+                       gpointer             user_data,
+                       GdkPixbuf           *mini_icon,
+                       GdkPixbuf           *icon)
 {
-  MetaFrameStyle *style;
   MetaThemeImplClass *impl_class;
+  MetaFrameStyle *style;
   MetaStyleInfo *style_info;
   gint title_height;
   PangoLayout *title_layout;
   MetaFrameGeometry fgeom;
+  gint i;
 
   g_return_if_fail (type < META_FRAME_TYPE_LAST);
 
-  style = get_frame_style (theme, type, flags);
+  impl_class = META_THEME_IMPL_GET_CLASS (theme->impl);
+  style = impl_class->get_frame_style (theme->impl, type, flags);
 
   /* Parser is not supposed to allow this currently */
   if (style == NULL)
     return;
 
-  impl_class = META_THEME_IMPL_GET_CLASS (theme->impl);
   style_info = get_style_info (theme, variant);
   title_height = get_title_height (theme, variant, type, flags);
   title_layout = create_title_layout (theme, variant, type, flags, title);
 
   impl_class->calc_geometry (theme->impl, style->layout, style_info,
                              title_height, flags, client_width, client_height,
-                             button_layout, type, &fgeom);
+                             theme->button_layout, type, &fgeom);
+
+  for (i = 0; i < 2; i++)
+    {
+      MetaButton *buttons;
+      gint n_buttons;
+      gint j;
+
+      if (i == 0)
+        {
+          buttons = theme->button_layout->left_buttons;
+          n_buttons = theme->button_layout->n_left_buttons;
+        }
+      else if (i == 1)
+        {
+          buttons = theme->button_layout->right_buttons;
+          n_buttons = theme->button_layout->n_right_buttons;
+        }
+      else
+        {
+          g_assert_not_reached ();
+        }
+
+      for (j = 0; j < n_buttons; j++)
+        {
+          MetaButton *button;
+          MetaButtonState state;
+          GdkRectangle rect;
+
+          button = &buttons[j];
+          state = META_BUTTON_STATE_NORMAL;
+          rect = button->rect.visible;
+
+          if (!button->visible || button->type == META_BUTTON_TYPE_SPACER ||
+              rect.width <= 0 || rect.height <= 0)
+            {
+              button->state = state;
+              continue;
+            }
+
+          if (func != NULL)
+            state = (* func) (button->type, button->rect.clickable, user_data);
+
+          g_assert (state >= META_BUTTON_STATE_NORMAL && state < META_BUTTON_STATE_LAST);
+          button->state = state;
+        }
+    }
 
   impl_class->draw_frame (theme->impl, style, style_info, cr, &fgeom,
-                          title_layout, flags, button_states, mini_icon, icon);
+                          title_layout, flags, theme->button_layout,
+                          mini_icon, icon);
 
   g_object_unref (title_layout);
 }
