@@ -40,13 +40,12 @@ struct _MetaSurfaceXRender
 
   Picture            picture;
 
-  Pixmap             mask_pixmap;
-  Picture            mask_picture;
-
   XserverRegion      border_clip;
 
   MetaShadowXRender *shadow;
   gboolean           shadow_changed;
+
+  gboolean           is_argb;
 };
 
 G_DEFINE_TYPE (MetaSurfaceXRender, meta_surface_xrender, META_TYPE_SURFACE)
@@ -87,46 +86,6 @@ shadow_changed (MetaSurfaceXRender *self)
   self->shadow_changed = TRUE;
 }
 
-static gboolean
-is_argb (MetaSurfaceXRender *self)
-{
-  MetaWindow *window;
-  Visual *xvisual;
-  XRenderPictFormat *format;
-
-  window = meta_surface_get_window (META_SURFACE (self));
-  xvisual = meta_window_get_toplevel_xvisual (window);
-
-  format = XRenderFindVisualFormat (self->xdisplay, xvisual);
-
-  if (format && format->type == PictTypeDirect && format->direct.alphaMask)
-    return TRUE;
-
-  if (window->opacity != OPAQUE)
-    return TRUE;
-
-  return FALSE;
-}
-
-static gboolean
-is_region_empty (Display       *xdisplay,
-                 XserverRegion  region)
-{
-  int n_rects;
-  XRectangle bounds;
-  XRectangle *rects;
-
-  rects = XFixesFetchRegionAndBounds (xdisplay, region, &n_rects, &bounds);
-
-  if (rects != NULL)
-    XFree (rects);
-
-  if (n_rects == 0 || bounds.width == 0 || bounds.height == 0)
-    return TRUE;
-
-  return FALSE;
-}
-
 static void
 paint_opaque_parts (MetaSurfaceXRender *self,
                     XserverRegion       paint_region,
@@ -148,7 +107,7 @@ paint_opaque_parts (MetaSurfaceXRender *self,
   shape_region = meta_surface_get_shape_region (surface);
   opaque_region = meta_surface_get_opaque_region (surface);
 
-  if ((is_argb (self) && opaque_region == None) ||
+  if ((self->is_argb && opaque_region == None) ||
       window->opacity != OPAQUE)
     return;
 
@@ -187,12 +146,6 @@ paint_opaque_parts (MetaSurfaceXRender *self,
   XFixesTranslateRegion (self->xdisplay, clip_region, x, y);
   XFixesIntersectRegion (self->xdisplay, clip_region, clip_region, paint_region);
 
-  if (is_region_empty (self->xdisplay, clip_region))
-    {
-      XFixesDestroyRegion (self->xdisplay, clip_region);
-      return;
-    }
-
   XFixesSetPictureClipRegion (self->xdisplay, paint_buffer, 0, 0, clip_region);
 
   XRenderComposite (self->xdisplay, PictOpSrc,
@@ -217,9 +170,6 @@ paint_argb_parts (MetaSurfaceXRender *self,
   XserverRegion shape_region;
   XserverRegion clip_region;
 
-  if (is_region_empty (self->xdisplay, self->border_clip))
-    return;
-
   surface = META_SURFACE (self);
 
   x = meta_surface_get_x (surface);
@@ -237,13 +187,10 @@ paint_argb_parts (MetaSurfaceXRender *self,
   XFixesIntersectRegion (self->xdisplay, border_clip, border_clip, clip_region);
   XFixesDestroyRegion (self->xdisplay, clip_region);
 
-  if (is_region_empty (self->xdisplay, border_clip))
-    return;
-
   XFixesSetPictureClipRegion (self->xdisplay, paint_buffer, 0, 0, border_clip);
 
   XRenderComposite (self->xdisplay, PictOpOver,
-                    self->picture, self->mask_picture, paint_buffer,
+                    self->picture, None, paint_buffer,
                     0, 0, 0, 0,
                     x, y, width, height);
 }
@@ -276,114 +223,6 @@ clip_to_shape_region (MetaSurfaceXRender *self,
   XFree (rects);
 }
 
-static Pixmap
-create_mask_pixmap (MetaSurfaceXRender *self,
-                    gboolean            with_opacity)
-{
-  MetaWindow *window;
-  MetaFrame *frame;
-  int width;
-  int height;
-  XRenderPictFormat *format;
-  double opacity;
-  Screen *xscreen;
-  cairo_surface_t *surface;
-  cairo_t *cr;
-  Pixmap pixmap;
-
-  window = meta_surface_get_window (META_SURFACE (self));
-
-  frame = meta_window_get_frame (window);
-  if (frame == NULL && window->opacity == OPAQUE)
-    return None;
-
-  width = frame != NULL ? meta_surface_get_width (META_SURFACE (self)) : 1;
-  height = frame != NULL ? meta_surface_get_height (META_SURFACE (self)) : 1;
-
-  format = XRenderFindStandardFormat (self->xdisplay, PictStandardA8);
-
-  meta_error_trap_push (self->display);
-  pixmap = XCreatePixmap (self->xdisplay,
-                          DefaultRootWindow (self->xdisplay),
-                          width,
-                          height,
-                          format->depth);
-
-  if (meta_error_trap_pop_with_return (self->display) != 0)
-    return None;
-
-  opacity = 1.0;
-  if (with_opacity)
-    opacity = (double) window->opacity / OPAQUE;
-
-  xscreen = DefaultScreenOfDisplay (self->xdisplay);
-  surface = cairo_xlib_surface_create_with_xrender_format (self->xdisplay,
-                                                           pixmap,
-                                                           xscreen,
-                                                           format,
-                                                           width,
-                                                           height);
-
-  cr = cairo_create (surface);
-
-  cairo_set_operator (cr, CAIRO_OPERATOR_CLEAR);
-  cairo_set_source_rgba (cr, 0, 0, 0, 1);
-  cairo_paint (cr);
-
-  if (frame != NULL)
-    {
-      cairo_rectangle_int_t rect;
-      cairo_region_t *frame_paint_region;
-      MetaFrameBorders borders;
-
-      rect.x = 0;
-      rect.y = 0;
-      rect.width = width;
-      rect.height = height;
-
-      frame_paint_region = cairo_region_create_rectangle (&rect);
-      meta_frame_calc_borders (frame, &borders);
-
-      rect.x += borders.total.left;
-      rect.y += borders.total.top;
-      rect.width -= borders.total.left + borders.total.right;
-      rect.height -= borders.total.top + borders.total.bottom;
-
-      cairo_region_subtract_rectangle (frame_paint_region, &rect);
-
-      cairo_rectangle (cr, rect.x, rect.y, rect.width, rect.height);
-      cairo_clip (cr);
-
-      cairo_set_operator (cr, CAIRO_OPERATOR_SOURCE);
-      cairo_set_source_rgba (cr, 0, 0, 0, opacity);
-      cairo_paint (cr);
-
-      cairo_reset_clip (cr);
-      gdk_cairo_region (cr, frame_paint_region);
-      cairo_region_destroy (frame_paint_region);
-      cairo_clip (cr);
-
-      cairo_push_group (cr);
-
-      cairo_set_operator (cr, CAIRO_OPERATOR_OVER);
-      meta_frame_get_mask (frame, cr);
-
-      cairo_pop_group_to_source (cr);
-      cairo_paint_with_alpha (cr, opacity);
-    }
-  else
-    {
-      cairo_set_operator (cr, CAIRO_OPERATOR_SOURCE);
-      cairo_set_source_rgba (cr, 0, 0, 0, opacity);
-      cairo_paint (cr);
-    }
-
-  cairo_destroy (cr);
-  cairo_surface_destroy (surface);
-
-  return pixmap;
-}
-
 static void
 free_picture (MetaSurfaceXRender *self)
 {
@@ -394,26 +233,6 @@ free_picture (MetaSurfaceXRender *self)
   self->picture = None;
 }
 
-static void
-free_mask_pixmap (MetaSurfaceXRender *self)
-{
-  if (self->mask_pixmap == None)
-    return;
-
-  XFreePixmap (self->xdisplay, self->mask_pixmap);
-  self->mask_pixmap = None;
-}
-
-static void
-free_mask_picture (MetaSurfaceXRender *self)
-{
-  if (self->mask_picture == None)
-    return;
-
-  XRenderFreePicture (self->xdisplay, self->mask_picture);
-  self->mask_picture = None;
-}
-
 static Picture
 get_window_picture (MetaSurfaceXRender *self)
 {
@@ -444,6 +263,8 @@ get_window_picture (MetaSurfaceXRender *self)
   if (pixmap == None)
     return None;
 
+  self->is_argb = format->type == PictTypeDirect && format->direct.alphaMask;
+
   pa.subwindow_mode = IncludeInferiors;
   pa_mask = CPSubwindowMode;
 
@@ -454,38 +275,11 @@ get_window_picture (MetaSurfaceXRender *self)
   return picture;
 }
 
-static Picture
-get_window_mask_picture (MetaSurfaceXRender *self)
-{
-  XRenderPictFormat *format;
-  Picture picture;
-
-  if (self->mask_pixmap == None)
-    return None;
-
-  format = XRenderFindStandardFormat (self->xdisplay, PictStandardA8);
-
-  meta_error_trap_push (self->display);
-
-  picture = XRenderCreatePicture (self->xdisplay,
-                                  self->mask_pixmap,
-                                  format,
-                                  0,
-                                  NULL);
-
-  meta_error_trap_pop (self->display);
-
-  return picture;
-}
-
 static void
 notify_appears_focused_cb (MetaWindow         *window,
                            GParamSpec         *pspec,
                            MetaSurfaceXRender *self)
 {
-  free_mask_pixmap (self);
-  free_mask_picture (self);
-
   shadow_changed (self);
 }
 
@@ -542,9 +336,6 @@ meta_surface_xrender_finalize (GObject *object)
 
   free_picture (self);
 
-  free_mask_pixmap (self);
-  free_mask_picture (self);
-
   if (self->border_clip != None)
     {
       XFixesDestroyRegion (self->xdisplay, self->border_clip);
@@ -562,8 +353,6 @@ meta_surface_xrender_get_image (MetaSurface *surface)
   MetaSurfaceXRender *self;
   Pixmap back_pixmap;
   MetaWindow *window;
-  Pixmap mask_pixmap;
-  gboolean free_pixmap;
   Visual *visual;
   int width;
   int height;
@@ -579,15 +368,6 @@ meta_surface_xrender_get_image (MetaSurface *surface)
 
   window = meta_surface_get_window (surface);
 
-  mask_pixmap = self->mask_pixmap;
-  free_pixmap = FALSE;
-
-  if (window->opacity != OPAQUE)
-    {
-      mask_pixmap = create_mask_pixmap (self, FALSE);
-      free_pixmap = TRUE;
-    }
-
   visual = meta_window_get_toplevel_xvisual (window);
   width = meta_surface_get_width (surface);
   height = meta_surface_get_height (surface);
@@ -609,51 +389,10 @@ meta_surface_xrender_get_image (MetaSurface *surface)
 
   clip_to_shape_region (self, cr);
 
-  if (mask_pixmap != None)
-    {
-      Screen *xscreen;
-      XRenderPictFormat *format;
-      MetaFrame *frame;
-      int mask_width;
-      int mask_height;
-      cairo_surface_t *mask;
-      cairo_pattern_t *pattern;
-
-      xscreen = DefaultScreenOfDisplay (self->xdisplay);
-      format = XRenderFindStandardFormat (self->xdisplay, PictStandardA8);
-
-      frame = meta_window_get_frame (window);
-      mask_width = frame != NULL ? width : 1;
-      mask_height = frame != NULL ? height : 1;
-
-      mask = cairo_xlib_surface_create_with_xrender_format (self->xdisplay,
-                                                            mask_pixmap,
-                                                            xscreen,
-                                                            format,
-                                                            mask_width,
-                                                            mask_height);
-
-      pattern = cairo_pattern_create_for_surface (mask);
-      cairo_surface_destroy (mask);
-
-      if (frame == NULL)
-        cairo_pattern_set_extend (pattern, CAIRO_EXTEND_REPEAT);
-
-      cairo_mask (cr, pattern);
-      cairo_pattern_destroy (pattern);
-
-      cairo_fill (cr);
-    }
-  else
-    {
-      cairo_paint (cr);
-    }
+  cairo_paint (cr);
 
   cairo_destroy (cr);
 
-  if (free_pixmap && mask_pixmap != None)
-    XFreePixmap (self->xdisplay, mask_pixmap);
-
   return image;
 }
 
@@ -670,14 +409,6 @@ meta_surface_xrender_is_visible (MetaSurface *surface)
 static void
 meta_surface_xrender_show (MetaSurface *surface)
 {
-  MetaSurfaceXRender *self;
-
-  self = META_SURFACE_XRENDER (surface);
-
-  /* The reason we free pixmap here is so that we will still have
-   * a valid pixmap when the window is unmapped.
-   */
-  free_mask_pixmap (self);
 }
 
 static void
@@ -688,7 +419,6 @@ meta_surface_xrender_hide (MetaSurface *surface)
   self = META_SURFACE_XRENDER (surface);
 
   free_picture (self);
-  free_mask_picture (self);
 
   shadow_changed (self);
 }
@@ -700,9 +430,6 @@ meta_surface_xrender_opacity_changed (MetaSurface *surface)
 
   self = META_SURFACE_XRENDER (surface);
 
-  free_mask_pixmap (self);
-  free_mask_picture (self);
-
   shadow_changed (self);
 }
 
@@ -749,9 +476,6 @@ meta_surface_xrender_free_pixmap (MetaSurface *surface)
   self = META_SURFACE_XRENDER (surface);
 
   free_picture (self);
-
-  free_mask_pixmap (self);
-  free_mask_picture (self);
 }
 
 static void
@@ -771,12 +495,6 @@ meta_surface_xrender_pre_paint (MetaSurface   *surface,
   if (self->picture == None)
     self->picture = get_window_picture (self);
 
-  if (self->mask_pixmap == None)
-    self->mask_pixmap = create_mask_pixmap (self, TRUE);
-
-  if (self->mask_picture == None)
-    self->mask_picture = get_window_mask_picture (self);
-
   if (self->shadow_changed)
     {
       if (self->shadow == NULL &&
