@@ -472,18 +472,18 @@ meta_core_get_active_workspace (Screen *xscreen)
 }
 
 void
-meta_core_show_window_menu (Display              *xdisplay,
-                            Window                frame_xwindow,
-                            const GdkRectangle   *rect,
-                            const GdkEventButton *event)
+meta_core_show_window_menu (Display            *xdisplay,
+                            Window              frame_xwindow,
+                            const GdkRectangle *rect,
+                            guint32             timestamp)
 {
   MetaWindow *window = get_window (xdisplay, frame_xwindow);
 
   if (meta_prefs_get_raise_on_click ())
     meta_window_raise (window);
-  meta_window_focus (window, event->time);
+  meta_window_focus (window, timestamp);
 
-  meta_window_show_menu (window, rect, (GdkEvent *) event);
+  meta_window_show_menu (window, rect, timestamp);
 }
 
 void
@@ -606,17 +606,13 @@ meta_core_get_menu_accelerator (MetaMenuOp           menu_op,
 
 const char*
 meta_core_get_workspace_name_with_index (Display *xdisplay,
-                                         Window   xroot,
                                          int      index)
 {
   MetaDisplay *display;
-  MetaScreen *screen;
   MetaWorkspace *workspace;
 
   display = meta_display_for_x_display (xdisplay);
-  screen = meta_display_screen_for_root (display, xroot);
-  g_assert (screen != NULL);
-  workspace = meta_screen_get_workspace_by_index (screen, index);
+  workspace = meta_screen_get_workspace_by_index (display->screen, index);
   return workspace ? meta_workspace_get_name (workspace) : NULL;
 }
 
