@@ -144,37 +144,8 @@ meta_window_delete (MetaWindow  *window,
                             delete_ping_reply_func,
                             delete_ping_timeout_func,
                             window);
-
-  if (window->has_focus)
-    {
-      /* FIXME Clean this up someday
-       * http://bugzilla.gnome.org/show_bug.cgi?id=108706
-       */
-#if 0
-      /* This is unfortunately going to result in weirdness
-       * if the window doesn't respond to the delete event.
-       * I don't know how to avoid that though.
-       */
-      meta_topic (META_DEBUG_FOCUS,
-                  "Focusing default window because focus window %s was deleted/killed\n",
-                  window->desc);
-      meta_workspace_focus_default_window (window->screen->active_workspace,
-                                           window);
-#else
-      meta_topic (META_DEBUG_FOCUS,
-                  "Not unfocusing %s on delete/kill\n",
-                  window->desc);
-#endif
-    }
-  else
-    {
-      meta_topic (META_DEBUG_FOCUS,
-                  "Window %s was deleted/killed but didn't have focus\n",
-                  window->desc);
-    }
 }
 
-
 void
 meta_window_kill (MetaWindow *window)
 {
