@@ -119,16 +119,6 @@ meta_print_compilation_info (void)
 #else
   meta_topic (META_DEBUG_XINERAMA, "Compiled without Xinerama extension\n");
 #endif
-#ifdef HAVE_XFREE_XINERAMA
-  meta_topic (META_DEBUG_XINERAMA, " (using XFree86 Xinerama)\n");
-#else
-  meta_topic (META_DEBUG_XINERAMA, " (not using XFree86 Xinerama)\n");
-#endif
-#ifdef HAVE_SOLARIS_XINERAMA
-  meta_topic (META_DEBUG_XINERAMA, " (using Solaris Xinerama)\n");
-#else
-  meta_topic (META_DEBUG_XINERAMA, " (not using Solaris Xinerama)\n");
-#endif
 #ifdef HAVE_RANDR
   meta_verbose ("Compiled with randr extension\n");
 #else
@@ -334,7 +324,9 @@ sigterm_handler (int signum)
 }
 
 static gboolean
-on_sigterm (void)
+on_sigterm (GIOChannel   *source,
+            GIOCondition  condition,
+            gpointer      user_data)
 {
   meta_quit ();
   return FALSE;
@@ -382,7 +374,7 @@ main (int argc, char **argv)
 
   channel = g_io_channel_unix_new (sigterm_pipe_fds[0]);
   g_io_channel_set_flags (channel, G_IO_FLAG_NONBLOCK, NULL);
-  g_io_add_watch (channel, G_IO_IN, (GIOFunc) on_sigterm, NULL);
+  g_io_add_watch (channel, G_IO_IN, on_sigterm, NULL);
   g_io_channel_set_close_on_unref (channel, TRUE);
   g_io_channel_unref (channel);
 
