@@ -485,7 +485,7 @@ handle_preference_init_string (void)
       if (cursor->handler)
         {
           if (cursor->target)
-            meta_bug ("%s has both a target and a handler\n", cursor->base.key);
+            g_error ("%s has both a target and a handler", cursor->base.key);
 
           g_settings_get_mapped (SETTINGS (cursor->base.schema),
                                  cursor->base.key, cursor->handler, NULL);
@@ -493,7 +493,7 @@ handle_preference_init_string (void)
       else
         {
           if (!cursor->target)
-            meta_bug ("%s must have handler or target\n", cursor->base.key);
+            g_error ("%s must have handler or target", cursor->base.key);
 
           if (*(cursor->target))
             g_free (*(cursor->target));
@@ -598,7 +598,7 @@ handle_preference_update_string (GSettings *settings,
   if (cursor->handler)
     {
       if (cursor->target)
-        meta_bug ("%s has both a target and a handler\n", cursor->base.key);
+        g_error ("%s has both a target and a handler", cursor->base.key);
 
       g_settings_get_mapped (SETTINGS (cursor->base.schema),
                              cursor->base.key, cursor->handler, NULL);
@@ -606,7 +606,7 @@ handle_preference_update_string (GSettings *settings,
   else
     {
       if (!cursor->target)
-        meta_bug ("%s must have handler or target\n", cursor->base.key);
+        g_error ("%s must have handler or target", cursor->base.key);
 
       value = g_settings_get_string (SETTINGS (cursor->base.schema),
                                      cursor->base.key);
@@ -686,7 +686,7 @@ meta_prefs_remove_listener (MetaPrefsChangedFunc func,
       tmp = tmp->next;
     }
 
-  meta_bug ("Did not find listener to remove\n");
+  g_error ("Did not find listener to remove");
 }
 
 static void
@@ -791,104 +791,6 @@ init_gtk_cursor_theme_size (void)
   gtk_cursor_theme_size_changed (settings, NULL, NULL);
 }
 
-static gboolean
-in_desktop (const gchar *name)
-{
-  const gchar *xdg_current_desktop;
-  gboolean result;
-  gchar **desktops;
-  gint i;
-
-  xdg_current_desktop = g_getenv ("XDG_CURRENT_DESKTOP");
-  if (xdg_current_desktop == NULL)
-    return FALSE;
-
-  result = FALSE;
-  desktops = g_strsplit (xdg_current_desktop, ":", -1);
-
-  for (i = 0; desktops[i] != NULL; i++)
-    {
-      if (g_strcmp0 (desktops[i], name) == 0)
-        {
-          result = TRUE;
-          break;
-        }
-    }
-
-  g_strfreev (desktops);
-
-  return result;
-}
-
-static void
-gtk_decoration_layout_changed (GtkSettings *settings,
-                               GParamSpec  *pspec,
-                               gpointer     user_data)
-{
-  gchar *layout;
-  gchar **sides;
-  gint i;
-  gint j;
-
-  g_object_get (settings, "gtk-decoration-layout", &layout, NULL);
-
-  sides = g_strsplit (layout, ":", -1);
-  g_free (layout);
-
-  for (i = 0; sides[i]; i++)
-    {
-      gchar **buttons;
-
-      buttons = g_strsplit (sides[i], ",", -1);
-
-      for (j = 0; buttons[j]; j++)
-        {
-          const gchar *button;
-
-          if (g_strcmp0 (buttons[j], "icon") == 0)
-            button = "menu";
-          else if (g_strcmp0 (buttons[j], "menu") == 0)
-            button = "appmenu";
-          else
-            button = NULL;
-
-          if (button)
-            {
-              g_free (buttons[j]);
-              buttons[j] = g_strdup (button);
-            }
-        }
-
-      g_free (sides[i]);
-      sides[i] = g_strjoinv (",", buttons);
-
-      g_strfreev (buttons);
-    }
-
-  layout = g_strjoinv (":", sides);
-  g_strfreev (sides);
-
-  update_button_layout (layout);
-  g_free (layout);
-}
-
-
-static void
-init_gtk_decoration_layout (void)
-{
-  GtkSettings *settings;
-
-  if (!in_desktop ("GNOME-Flashback"))
-    return;
-
-  settings = gtk_settings_get_default ();
-
-  g_signal_connect (settings, "notify::gtk-decoration-layout",
-                    G_CALLBACK (gtk_decoration_layout_changed), NULL);
-
-  gtk_decoration_layout_changed (settings, NULL, NULL);
-}
-
 static void
 gtk_theme_name_changed (GtkSettings *settings,
                         GParamSpec  *pspec,
@@ -953,7 +855,6 @@ meta_prefs_init (void)
   init_workspace_names ();
 
   init_gtk_cursor_theme_size ();
-  init_gtk_decoration_layout ();
   init_gtk_theme_name ();
 }
 
@@ -1040,8 +941,8 @@ maybe_give_disable_workarounds_warning (void)
     {
       first_disable = FALSE;
 
-      meta_warning (_("Workarounds for broken applications disabled. "
-                      "Some applications may not behave properly.\n"));
+      g_warning ("Workarounds for broken applications disabled. "
+                 "Some applications may not behave properly.");
     }
 }
 
@@ -1117,10 +1018,8 @@ titlebar_handler (GVariant *value,
 
   if (desc == NULL)
     {
-      meta_warning (_("Could not parse font description "
-                      "\"%s\" from GSettings key %s\n"),
-                    string_value ? string_value : "(null)",
-                    KEY_TITLEBAR_FONT);
+      g_warning ("Could not parse font description \"%s\" from GSettings key %s",
+                 string_value ? string_value : "(null)", KEY_TITLEBAR_FONT);
 
       return FALSE;
     }
@@ -1176,12 +1075,8 @@ mouse_button_mods_handler (GVariant *value,
 
   if (!string_value || !meta_ui_parse_modifier (string_value, &mods))
     {
-      meta_topic (META_DEBUG_KEYBINDINGS,
-                  "Failed to parse new GSettings value\n");
-
-      meta_warning (_("\"%s\" found in configuration database is "
-                      "not a valid value for mouse button modifier\n"),
-                    string_value);
+      g_warning ("\"%s\" found in configuration database is not a valid value "
+                 "for mouse button modifier", string_value);
 
       return FALSE;
     }
@@ -1220,9 +1115,6 @@ button_layout_handler (GVariant *value,
 
   *result = NULL; /* ignored */
 
-  if (in_desktop ("GNOME-Flashback"))
-    return TRUE;
-
   string_value = g_variant_get_string (value, NULL);
 
   if (string_value)
@@ -1252,7 +1144,6 @@ meta_prefs_get_disable_workarounds (void)
   return disable_workarounds;
 }
 
-#ifdef WITH_VERBOSE_MODE
 const char*
 meta_preference_to_string (MetaPreference pref)
 {
@@ -1361,7 +1252,6 @@ meta_preference_to_string (MetaPreference pref)
 
   return "(unknown)";
 }
-#endif /* WITH_VERBOSE_MODE */
 
 void
 meta_prefs_set_num_workspaces (int n_workspaces)
@@ -1415,8 +1305,7 @@ update_binding (MetaKeyPref *binding,
   /* Okay, so, we're about to provide a new list of key combos for this
    * action. Delete any pre-existing list.
    */
-  g_slist_foreach (binding->bindings, (GFunc) g_free, NULL);
-  g_slist_free (binding->bindings);
+  g_slist_free_full (binding->bindings, g_free);
   binding->bindings = NULL;
 
   for (i = 0; strokes && strokes[i]; i++)
@@ -1429,8 +1318,8 @@ update_binding (MetaKeyPref *binding,
         {
           meta_topic (META_DEBUG_KEYBINDINGS,
                       "Failed to parse new GSettings value\n");
-          meta_warning (_("\"%s\" found in configuration database is not a valid value for keybinding \"%s\"\n"),
-                        strokes[i], binding->name);
+          g_warning ("\"%s\" found in configuration database is not a valid "
+                     "value for keybinding \"%s\"", strokes[i], binding->name);
 
           /* Value is kept and will thus be removed next time we save the key.
            * Changing the key in response to a modification could lead to cyclic calls. */
@@ -1445,10 +1334,8 @@ update_binding (MetaKeyPref *binding,
           0 != keysym &&
           (META_VIRTUAL_SHIFT_MASK == mods || 0 == mods))
         {
-          meta_warning ("Cannot bind \"%s\" to %s: it needs a modifier "
-                        "such as Ctrl or Alt.\n",
-                        binding->name,
-                        strokes[i]);
+          g_warning ("Cannot bind \"%s\" to %s: it needs a modifier "
+                     "such as Ctrl or Alt.", binding->name, strokes[i]);
 
           /* Value is kept and will thus be removed next time we save the key.
            * Changing the key in response to a modification could lead to cyclic calls. */
@@ -1627,7 +1514,7 @@ meta_prefs_add_keybinding (const char           *name,
 
   if (g_hash_table_lookup (key_bindings, name))
     {
-      meta_warning ("Trying to re-add keybinding \"%s\".\n", name);
+      g_warning ("Trying to re-add keybinding \"%s\".", name);
       return FALSE;
     }
 
@@ -1769,35 +1656,6 @@ meta_prefs_get_window_binding (const char          *name,
 gboolean
 meta_prefs_get_compositing_manager (void)
 {
-  MetaDisplay *display;
-  static gboolean warned = FALSE;
-
-  display = meta_get_display ();
-
-  if (!display)
-    return compositing_manager;
-
-  if (!compositing_manager)
-    warned = FALSE;
-
-  if (compositing_manager &&
-      (!META_DISPLAY_HAS_COMPOSITE (display) ||
-       !META_DISPLAY_HAS_DAMAGE (display) ||
-       !META_DISPLAY_HAS_XFIXES (display) ||
-       !META_DISPLAY_HAS_RENDER (display)))
-    {
-      if (!warned)
-        {
-          meta_warning (_("Missing %s extension required for compositing\n"),
-                        !META_DISPLAY_HAS_COMPOSITE (display) ? "composite" :
-                        !META_DISPLAY_HAS_DAMAGE (display) ? "damage" :
-                        !META_DISPLAY_HAS_XFIXES (display) ? "xfixes" : "render");
-          warned = TRUE;
-        }
-
-      return FALSE;
-    }
-
   return compositing_manager;
 }
 
