@@ -75,7 +75,7 @@ meta_window_release_saved_state (const MetaWindowSessionInfo *info)
 static void ice_io_error_handler (IceConn connection);
 
 static void new_ice_connection (IceConn connection, IcePointer client_data,
-				Bool opening, IcePointer *watch_data);
+                                Bool opening, IcePointer *watch_data);
 
 static void        save_state         (void);
 static char*       load_state         (const char *previous_save_file);
@@ -110,7 +110,7 @@ process_ice_messages (GIOChannel *channel,
        * being cleaned up, since it is owned by libSM.
        */
       disconnect ();
-      meta_quit (META_EXIT_SUCCESS);
+      meta_quit ();
 
       return FALSE;
     }
@@ -122,7 +122,7 @@ process_ice_messages (GIOChannel *channel,
    the ICE connection to be handled via the event loop.  */
 static void
 new_ice_connection (IceConn connection, IcePointer client_data, Bool opening,
-		    IcePointer *watch_data)
+                    IcePointer *watch_data)
 {
   guint input_id;
 
@@ -179,7 +179,7 @@ ice_init (void)
       default_handler = IceSetIOErrorHandler (ice_io_error_handler);
 
       if (ice_installed_handler == default_handler)
-	ice_installed_handler = NULL;
+        ice_installed_handler = NULL;
 
       IceAddConnectionWatch (new_ice_connection, NULL);
 
@@ -292,7 +292,7 @@ meta_session_init (const char *previous_client_id,
   else
     {
       if (client_id == NULL)
-        meta_bug ("Session manager gave us a NULL client ID?");
+        g_error ("Session manager gave us a NULL client ID?");
       meta_topic (META_DEBUG_SM, "Obtained session ID '%s'\n", client_id);
     }
 
@@ -503,13 +503,13 @@ save_yourself_callback (SmcConn   smc_conn,
       /* Double check that this is a section 7.2 SaveYourself: */
 
       if (save_style == SmSaveLocal &&
-	  interact_style == SmInteractStyleNone &&
-	  !shutdown && !fast)
-	{
-	  /* The protocol requires this even if xsm ignores it. */
-	  SmcSaveYourselfDone (session_connection, successful);
-	  return;
-	}
+          interact_style == SmInteractStyleNone &&
+          !shutdown && !fast)
+        {
+          /* The protocol requires this even if xsm ignores it. */
+          SmcSaveYourselfDone (session_connection, successful);
+          return;
+        }
     }
 #endif
 
@@ -538,13 +538,12 @@ save_yourself_callback (SmcConn   smc_conn,
   save_yourself_possibly_done (shutdown, successful);
 }
 
-
 static void
 die_callback (SmcConn smc_conn, SmPointer client_data)
 {
   meta_topic (META_DEBUG_SM, "Exiting at request of session manager\n");
   disconnect ();
-  meta_quit (META_EXIT_SUCCESS);
+  meta_quit ();
 }
 
 static void
@@ -701,6 +700,20 @@ window_type_to_string (MetaWindowType type)
       return "splashscreen";
     case META_WINDOW_UTILITY:
       return "utility";
+    case META_WINDOW_DROPDOWN_MENU:
+      return "dropdown_menu";
+    case META_WINDOW_POPUP_MENU:
+      return "popup_menu";
+    case META_WINDOW_TOOLTIP:
+      return "tooltip";
+    case META_WINDOW_NOTIFICATION:
+      return "notification";
+    case META_WINDOW_COMBO:
+      return "combo";
+    case META_WINDOW_DND:
+      return "dnd";
+    case META_WINDOW_OVERRIDE_OTHER:
+      return "override_redirect";
     default:
       break;
     }
@@ -729,6 +742,20 @@ window_type_from_string (const char *str)
     return META_WINDOW_UTILITY;
   else if (strcmp (str, "splashscreen") == 0)
     return META_WINDOW_SPLASHSCREEN;
+  else if (strcmp (str, "dropdown_menu") == 0)
+    return META_WINDOW_DROPDOWN_MENU;
+  else if (strcmp (str, "popup_menu") == 0)
+    return META_WINDOW_POPUP_MENU;
+  else if (strcmp (str, "tooltip") == 0)
+    return META_WINDOW_TOOLTIP;
+  else if (strcmp (str, "notification") == 0)
+    return META_WINDOW_NOTIFICATION;
+  else if (strcmp (str, "combo") == 0)
+    return META_WINDOW_COMBO;
+  else if (strcmp (str, "dnd") == 0)
+    return META_WINDOW_DND;
+  else if (strcmp (str, "override_redirect") == 0)
+    return META_WINDOW_OVERRIDE_OTHER;
   else
     return META_WINDOW_NORMAL;
 }
@@ -839,15 +866,15 @@ save_state (void)
   if (mkdir (metacity_dir, 0700) < 0 &&
       errno != EEXIST)
     {
-      meta_warning (_("Could not create directory '%s': %s\n"),
-                    metacity_dir, g_strerror (errno));
+      g_warning ("Could not create directory '%s': %s",
+                 metacity_dir, g_strerror (errno));
     }
 
   if (mkdir (session_dir, 0700) < 0 &&
       errno != EEXIST)
     {
-      meta_warning (_("Could not create directory '%s': %s\n"),
-                    session_dir, g_strerror (errno));
+      g_warning ("Could not create directory '%s': %s",
+                 session_dir, g_strerror (errno));
     }
 
   meta_topic (META_DEBUG_SM, "Saving session to '%s'\n", full_save_file ());
@@ -856,8 +883,8 @@ save_state (void)
 
   if (outfile == NULL)
     {
-      meta_warning (_("Could not open session file '%s' for writing: %s\n"),
-                    full_save_file (), g_strerror (errno));
+      g_warning ("Could not open session file '%s' for writing: %s",
+                 full_save_file (), g_strerror (errno));
       goto out;
     }
 
@@ -880,7 +907,7 @@ save_state (void)
   fprintf (outfile, "<metacity_session id=\"%s\">\n",
            client_id);
 
-  windows = meta_display_list_windows (meta_get_display ());
+  windows = meta_display_list_windows (meta_get_display (), META_LIST_DEFAULT);
   stack_position = 0;
 
   windows = g_slist_sort (windows, meta_display_stack_cmp);
@@ -997,13 +1024,13 @@ save_state (void)
       /* FIXME need a dialog for this */
       if (ferror (outfile))
         {
-          meta_warning (_("Error writing session file '%s': %s\n"),
-                        full_save_file (), g_strerror (errno));
+          g_warning ("Error writing session file '%s': %s",
+                     full_save_file (), g_strerror (errno));
         }
       if (fclose (outfile))
         {
-          meta_warning (_("Error closing session file '%s': %s\n"),
-                        full_save_file (), g_strerror (errno));
+          g_warning ("Error closing session file '%s': %s",
+                     full_save_file (), g_strerror (errno));
         }
     }
 
@@ -1132,8 +1159,7 @@ load_state (const char *previous_save_file)
 
  error:
 
-  meta_warning (_("Failed to parse saved session file: %s\n"),
-                error->message);
+  g_warning ("Failed to parse saved session file: %s", error->message);
   g_error_free (error);
 
   if (parse_data.info)
@@ -1544,7 +1570,7 @@ get_possible_matches (MetaWindow *window)
         }
       else
         {
-          if (meta_is_verbose ())
+          if (meta_check_debug_flags (META_DEBUG_SM))
             {
               if (!both_null_or_matching (info->id, window->sm_client_id))
                 meta_topic (META_DEBUG_SM, "Window %s has SM client ID %s, saved state has %s, no match\n",
@@ -1758,7 +1784,7 @@ warn_about_lame_clients_and_finish_interact (gboolean shutdown)
   GSList *columns = NULL;
   GPid pid;
 
-  windows = meta_display_list_windows (meta_get_display ());
+  windows = meta_display_list_windows (meta_get_display (), META_LIST_DEFAULT);
   tmp = windows;
   while (tmp != NULL)
     {
