@@ -646,12 +646,14 @@ ensure_work_areas_validated (MetaWorkspace *workspace)
       workspace->monitor_region[i] =
         meta_rectangle_get_minimal_spanning_set_for_region (
           &workspace->screen->monitor_infos[i].rect,
-          workspace->all_struts);
+          workspace->all_struts,
+          FALSE);
     }
   workspace->screen_region =
     meta_rectangle_get_minimal_spanning_set_for_region (
       &workspace->screen->rect,
-      workspace->all_struts);
+      workspace->all_struts,
+      TRUE);
 
   /* STEP 3: Get the work areas (region-to-maximize-to) for the screen and
    *         monitors.
