@@ -24,17 +24,8 @@
 
 #include <glib.h>
 
-typedef enum
-{
-  META_EXIT_SUCCESS,
-  META_EXIT_ERROR
-} MetaExitCode;
-
-/* exit immediately */
-void meta_exit (MetaExitCode code) __attribute__((noreturn));
-
 /* g_main_loop_quit() then fall out of main() */
-void meta_quit (MetaExitCode code);
+void meta_quit (void);
 
 void meta_restart (void);
 
