@@ -529,7 +529,7 @@ splashscreen_cb (GSimpleAction *action,
   GtkWidget *vbox;
 
   window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
-  set_gtk_window_type (GTK_WINDOW (window), "_NET_WM_WINDOW_TYPE_SPLASHSCREEN");
+  set_gtk_window_type (GTK_WINDOW (window), "_NET_WM_WINDOW_TYPE_SPLASH");
   gtk_window_set_title (GTK_WINDOW (window), "Splashscreen");
 
   vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 0);
@@ -560,6 +560,8 @@ make_dock (int type)
   GtkWidget *image;
   GtkWidget *box;
   GtkWidget *button;
+  GdkScreen *screen;
+  GdkWindow *root;
 
   g_return_if_fail (type != DOCK_ALL);
 
@@ -575,6 +577,7 @@ make_dock (int type)
       box = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
       break;
     case DOCK_ALL:
+    default:
       break;
     }
 
@@ -594,6 +597,9 @@ make_dock (int type)
 
   gtk_container_add (GTK_CONTAINER (window), box);
 
+  screen = gdk_screen_get_default ();
+  root = gdk_screen_get_root_window (screen);
+
 #define DOCK_SIZE 48
   switch (type)
     {
@@ -605,7 +611,7 @@ make_dock (int type)
       break;
     case DOCK_RIGHT:
       gtk_widget_set_size_request (window, DOCK_SIZE, 400);
-      gtk_window_move (GTK_WINDOW (window), gdk_screen_width () - DOCK_SIZE, 200);
+      gtk_window_move (GTK_WINDOW (window), gdk_window_get_width (root) - DOCK_SIZE, 200);
       set_gtk_window_struts (window, 0, DOCK_SIZE, 0, 0);
       gtk_window_set_title (GTK_WINDOW (window), "RightDock");
       break;
@@ -617,11 +623,12 @@ make_dock (int type)
       break;
     case DOCK_BOTTOM:
       gtk_widget_set_size_request (window, 600, DOCK_SIZE);
-      gtk_window_move (GTK_WINDOW (window), 200, gdk_screen_height () - DOCK_SIZE);
+      gtk_window_move (GTK_WINDOW (window), 200, gdk_window_get_height (root) - DOCK_SIZE);
       set_gtk_window_struts (window, 0, 0, 0, DOCK_SIZE);
       gtk_window_set_title (GTK_WINDOW (window), "BottomDock");
       break;
     case DOCK_ALL:
+    default:
       break;
     }
 
@@ -689,15 +696,21 @@ desktop_cb (GSimpleAction *action,
             GVariant      *parameter,
             gpointer       callback_data)
 {
+  GdkScreen *screen;
+  GdkWindow *root;
   GtkWidget *window;
   GtkWidget *label;
   GdkRGBA    desktop_color;
 
+  screen = gdk_screen_get_default ();
+  root = gdk_screen_get_root_window (screen);
+
   window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
   set_gtk_window_type (GTK_WINDOW (window), "_NET_WM_WINDOW_TYPE_DESKTOP");
   gtk_window_set_title (GTK_WINDOW (window), "Desktop");
   gtk_widget_set_size_request (window,
-                               gdk_screen_width (), gdk_screen_height ());
+                               gdk_window_get_width (root),
+                               gdk_window_get_height (root));
   gtk_window_move (GTK_WINDOW (window), 0, 0);
 
   desktop_color.red = 0.32;
@@ -983,6 +996,9 @@ do_appwindow (GSimpleAction *action,
   GtkTextBuffer *buffer;
   GSimpleActionGroup *action_group;
   GtkBuilder *builder;
+  GMenuModel *model;
+  GtkWidget *menubar;
+  GtkWidget *toolbar;
 
   /* Create the toplevel window
    */
@@ -1016,15 +1032,15 @@ do_appwindow (GSimpleAction *action,
   /* Create the menubar
    */
 
-  GMenuModel *model = G_MENU_MODEL (gtk_builder_get_object (builder, "menubar"));
-  GtkWidget *menubar = gtk_menu_bar_new_from_model (model);
+  model = G_MENU_MODEL (gtk_builder_get_object (builder, "menubar"));
+  menubar = gtk_menu_bar_new_from_model (model);
   gtk_grid_attach (GTK_GRID (grid), menubar, 0, 0, 1, 1);
   gtk_widget_set_hexpand (menubar, TRUE);
 
   /* Create the toolbar
    */
 
-  GtkWidget *toolbar = create_toolbar ();
+  toolbar = create_toolbar ();
   gtk_grid_attach (GTK_GRID (grid), toolbar, 0, 1, 1, 1);
   gtk_widget_set_hexpand (toolbar, TRUE);
 
